"""Query Valve Main and Game Servers.

This package provides functions to 1) search Valve's MAIN server for list
of GAME servers, and 2) query GAME servers for current map, number of
players, etc.
"""
