<!doctype html>
<html lang="en">
    <head>
        {% block head %}
        <meta charset="utf-8">
        <meta name="viewport" content="width=device-width, initial-scale=1, shrink-to-fit=no">
            {% block styles %}
                {{ bootstrap.load_css() }}
            {% endblock %}
        {% endblock %}

        {% block title %}{% endblock %}
    </head>
    <body>
        {% block content %}{% endblock %}

        {% block scripts %}
            {{ bootstrap.load_js() }}
        {% endblock %}
    </body>
</html>
